import re

from django import apps
from django.conf import settings

from .registry import ElementRegistry


class ScriptMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response
        self.app = apps.apps.get_app_config('djwc')

    def __call__(self, request):
        response = self.get_response(request)

        if not getattr(response, 'content', ''):
            return response

        loaded = []
        for tag in re.findall(b'</(?P<t>[a-z]*-[a-z]*)>', response.content):
            if tag in loaded:
                continue

            if tag not in self.app.bcomponents:
                print(f'Element {tag} not found in registry')
                continue

            response.content = response.content.replace(
                b'</head>',
                b''.join([
                    b'<script type="module">import ',
                    b"'",
                    self.app.bstatic,
                    self.app.bcomponents[tag].encode('utf8'),
                    b"'",
                    b'</script>',
                    b'</head>',
                ])
            )
            loaded.append(tag)

        return response



import importlib
from pathlib import Path

from django import apps
from django.conf import settings


class AppConfig(apps.AppConfig):
    name = 'djwc'
    verbose_name = 'Django WebComponents'

    def ready(self):
        DJWC = getattr(settings, 'DJWC', {})

        self.static = (Path(__file__).parent / 'static/djwc').absolute()
        self.bstatic = f'{settings.STATIC_URL}djwc/'.encode('utf8')

        self.components = dict()

        for name, config in apps.apps.app_configs.items():
            if 'polymer' in name:
                breakpoint()
            self.components.update(getattr(config, 'components', {}))

        for lib in DJWC.get('LIBRARIES', []):
            self.components.update(importlib.import_module(lib).components)

        if 'COMPONENTS' in DJWC:
            self.components.update(settings.DJWC['COMPONENTS'])

        self.bcomponents = {
            k.encode('utf8'): v for k, v in self.components.items()
        }



from django.contrib import admin
from django.views import generic
from django.urls import path

urlpatterns = [
    path('', generic.TemplateView.as_view(template_name='base.html')),
    path('admin/', admin.site.urls),
]
